package BSS.Controllers;

import BSS.*;

import java.util.*;

public class DoorController extends BaseController
{
    private int _y1;
    private LinkedList _query1;
    final static Set A1_Acceptable_Event = new TreeSet();
    final static Set A1_Acceptable_Message = new TreeSet();

    private int _y2;
    private int _y2_1;
    private LinkedList _query2;

    private int _MC_id;

    static int MAX_PIN_FAIL = 5;
    private String _PIN;
    private String _input_PIN;

    static
    {
        A1_Acceptable_Event.add(new Integer(EVENT_PUSH));

        A1_Acceptable_Message.add(new Integer(MSG_LOCK_DOOR));
        A1_Acceptable_Message.add(new Integer(MSG_UNLOCK_DOOR));
    }

    //--- Public section ---
    public DoorController(int id, Logger logger, Device device)
    {
        super(id, logger, device);
        _query1 = new LinkedList();
        _query2 = new LinkedList();
    }

    public String toString()
    {
        return "Door#" + ID();
    }

    //--- Controller interface methods ---
    public void init()
    {
        _query1.clear();
        _query2.clear();
        _y1 = 2;
        _y2 = 0;
        _y2_1 = 0;
        Map conf = sysLoadConfig();
        _MC_id = ((Integer)conf.get("MC")).intValue();
        _PIN = (String)conf.get("PIN");
    }

    public void run()
    {
        if (!_query2.isEmpty())
        {
            Integer event = (Integer)_query2.removeFirst();
            A2(event.intValue());
        }
        else if (needCheck_A2())
        {
            A2(0);
        }

        if (!_query1.isEmpty())
        {
            Integer event = (Integer)_query1.removeFirst();
            A1(event.intValue());
        }
        else if (needCheck_A1())
        {
            A1(0);
        }
    }

    public void envOnEvent(int event_type)
    {
        if (A1_Acceptable_Event.contains(new Integer(event_type)))
        {
            _query1.addLast(new Integer(event_type));
        }
    }

    public Message msgOnReceive(Message msg)
    {
        Message response = null;

        if (REQ_ENTRY_IS_LOCKED == msg.code)
        {
            response = msgMakeReply(msg, isLocked()? 1 : 0);
        }
        else if (REQ_ENTRY_IS_CLOSED == msg.code)
        {
            response = msgMakeReply(msg, isClosed()? 1 : 0);
        }
        else if (A1_Acceptable_Message.contains(new Integer(msg.code)))
        {
            _query1.addLast(new Integer(msg.code));
        }

        return response;
    }

    //--- Automaton methods ---
    private void A1(int e)
    {
        log_start("A1", _y1, e);
        int y_now = _y1;

        switch (y_now)
        {
          case 0:
            if (e == 1)
            {                                               _y1 = 3; }
            else
            if (x1())
            { z2();                                         _y1 = 1; }
          break;

          case 1:
            if (!x1())
            { z1();                                         _y1 = 0; }
            else
            if (e == 1 || _y2 == 3)
            {                                               _y1 = 2; }
          break;

          case 2:
            if (e == 2 || _y2 == 3)
            { z5(); z4();                                   _y1 = 1; }
            else
            if (e == 3)
            { z6();                                                  }
          break;

          case 3:
            if (e == 2)
            {                                               _y1 = 0; }
            else
            if (x1())
            { z2();                                         _y1 = 2; }
          break;
        }

        if (_y1 != y_now)
        {
            switch (_y1)
            {
              case 2:
                z5(); z3();
              break;
            }
        }

        log_end("A1", _y1);
    }

    private void A2(int e)
    {
        log_start("A2", _y2, e);
        int y_now = _y2;

        switch (y_now)
        {
          case 0:
            if (x4())
            {                                               _y2 = 1; }
          break;

          case 1:
            if (x5())
            {                                               _y2 = 2; }
          break;

          case 2:
            boolean xx6 = x6();
            boolean xx7 = x7();
            if (xx6)
            {                                               _y2 = 0; }
            else
            if (xx6 && xx7)
            {                                               _y2 = 3; }
            else
            if (xx6 && !xx7)
            {                                               _y2 = 4; }
          break;

          case 3:
            if (e == 11)
            { z11();                                        _y2 = 0; }
          break;

          case 4:
            boolean xx8 = x8();
            if (xx8)
            { z11(); z6();                                  _y2 = 0; }
            else
            if (!xx8)
            {                                               _y2 = 0; }
          break;
        }

        if (_y2 != y_now)
        {
            switch (_y2)
            {
              case 0:
                z8();
              break;

              case 1:
                z7();
              break;

              case 4:
                z10();
              break;
            }
        }

        log_end("A2", _y2);
    }

    private boolean x1()
    {
        boolean res = envHasContact(1);
        log_x("x1", res, " ");
        return res;
    }

    private boolean x4()
    {
        boolean res =  envSmartCardInserted(1);
        log_x("x4", res, " -");
        return res;
    }

    private boolean x5()
    {
        boolean res =  envSmartCardRead(1);
        log_x("x5", res, " - ");
        return res;
    }

    private boolean x6()
    {
        boolean res = true;

        int len = _input_PIN.length();
        if (1 > len)
        {
            res = false;
        }
        else
        {
            for (int i = 0; i < len; i++)
            {
                char ch = _input_PIN.charAt(i);
                if ('0' > ch || '9' < ch)
                {
                    res = false;
                    break;
                }
            }
        }

        log_x("x6", res, "   -");
        return res;
    }

    private boolean x7()
    {
        boolean res =  _PIN.equals(_input_PIN);
        log_x("x7", res, " PIN- -");
        return res;
    }

    private boolean x8()
    {
        boolean res =  MAX_PIN_FAIL < _y2_1;
        log_x("x8", res, " PIN-");
        return res;
    }

    private void z1()
    {
        log_z("z1", " ");
        envOn_LED(1);
    }

    private void z2()
    {
        log_z("z2", " ");
        envOff_LED(1);
    }

    private void z3()
    {
        log_z("z3", " ");
        envLock(1);
    }

    private void z4()
    {
        log_z("z4", " ");
        envUnlock(1);
    }

    private void z5()
    {
        log_z("z5", "  ");
        _query2.addLast(new Integer(11));
    }

    private void z6()
    {
        log_z("z6", "  ");
        msgSend(_MC_id, MSG_BURGLARY);
    }

    private void z7()
    {
        log_z("z7", "  -");
        _input_PIN = envReadSmartCard(1);
    }

    private void z8()
    {
        log_z("z8", " ");
        envResetSmartCardDrive(1);
    }

    private void z10()
    {
        log_z("z10", " ");
        _y2_1++;
    }

    private void z11()
    {
        log_z("z11", " ");
        _y2_1 = 0;
    }

    //--- Methods for private usage ---
    private boolean isClosed()
    {
        return (1 == _y1) || (2 == _y1);
    }

    private boolean isLocked()
    {
        return (_y1 == 2);
    }

    private boolean needCheck_A1()
    {
        return (0 == _y1) || (1 == _y1) || (3 == _y1);
    }

    private boolean needCheck_A2()
    {
        return (0 == _y2) || (1 == _y2) || (2 == _y2) || (4 == _y2);
    }
}
