package BSS.Controllers;

import BSS.*;

import java.util.*;

public class GatewayController extends BaseController
{
    private int _y3;
    private LinkedList _query3;
    final static Set Acceptable_Event = new TreeSet();
    final static Set Acceptable_Message = new TreeSet();

    private int _MC_id;
    private boolean _hasPhotoCell;

    static
    {
        Acceptable_Event.add(new Integer(EVENT_PUSH));

        Acceptable_Message.add(new Integer(MSG_CLOSE_GATEWAY));
        Acceptable_Message.add(new Integer(MSG_OPEN_GATEWAY));
    }

    //--- Public section ---
    public GatewayController(int id, Logger logger, Device device)
    {
        super(id, logger, device);
        _query3 = new LinkedList();
    }

    public String toString()
    {
        return "Gateway#" + ID();
    }

    //--- Controller interface methods ---
    public void init()
    {
        _query3.clear();
        _y3 = 0;
        Map conf = sysLoadConfig();
        _MC_id = ((Integer)conf.get("MC")).intValue();
        _hasPhotoCell = ((Boolean)conf.get("PhotoCell")).booleanValue();
    }

    public void run()
    {
        if (!_query3.isEmpty())
        {
            Integer event = (Integer)_query3.removeFirst();
            A3(event.intValue());
        }
        else if (needCheck_A3())
        {
            A3(0);
        }
    }

    public void envOnEvent(int event_type)
    {
        if (Acceptable_Event.contains(new Integer(event_type)))
        {
            _query3.addLast(new Integer(event_type));
        }
    }

    public Message msgOnReceive(Message msg)
    {
        Message response = null;

        if (REQ_ENTRY_IS_LOCKED == msg.code)
        {
            response = msgMakeReply(msg, isLocked()? 1 : 0);
        }
        else if (REQ_ENTRY_IS_CLOSED == msg.code)
        {
            response = msgMakeReply(msg, isClosed()? 1 : 0);
        }
        else if (Acceptable_Message.contains(new Integer(msg.code)))
        {
            _query3.addLast(new Integer(msg.code));
        }

        return response;
    }

    //--- Automaton methods ---
    private void A3(int e)
    {
        log_start("A3", _y3, e);
        int y_now = _y3;

        switch (y_now)
        {
          case 0:
            boolean xx9 = x9();
            if (e == 3)
            { z6();                                                  }
            else
            if (e == 5 && !xx9)
            { z16();                                        _y3 = 3; }
            else
            if (e == 5 && xx9)
            { z16();                                        _y3 = 4; }
          break;

          case 1:
            if (e == 4)
            {                                               _y3 = 2; }
            else
            if (x9() && !x10())
            {                                               _y3 = 5; }
          break;

          case 2:
            if (e == 5)
            { z12();                                        _y3 = 3; }
            if (x11())
            { z12();                                        _y3 = 0; }
          break;

          case 3:
            if (e == 4)
            { z12();                                        _y3 = 2; }
            else
            if (x9() && !x10())
            { z12();                                        _y3 = 5; }
            else
            if (x14())
            {                                               _y3 = 1; }
          break;

          case 4:
            if (e == 4)
            {                                               _y3 = 0; }
            else
            if (x10())
            {                                               _y3 = 3; }
          break;

          case 5:
            if (e == 4)
            { z12();                                        _y3 = 2; }
            else
            if (x10())
            { z12();                                        _y3 = 3; }
            else
            if (x11())
            { z12();                                        _y3 = 4; }
          break;
        }

        if (_y3 != y_now)
        {
            switch (_y3)
            {
              case 0:
                z15();
              break;

              case 1:
                z12();
              break;

              case 2:
                z13();
              break;

              case 3:
                z14();
              break;

              case 5:
                z13();
              break;
            }
        }

        log_end("A3", _y3);
    }

    private boolean x9()
    {
        boolean res = _hasPhotoCell;
        log_x("x9", res, " ");
        return res;
    }

    private boolean x10()
    {
        boolean res =  envPhotoDark(1);
        log_x("x10", res, "");
        return res;
    }

    private boolean x11()
    {
        boolean res = envHasContact(1);
        log_x("x11", res, " ");
        return res;
    }

    private boolean x14()
    {
        boolean res = envHasContact(2);
        log_x("x14", res, "  ");
        return res;
    }

    private void z6()
    {
        log_z("z6", "  ");
        msgSend(_MC_id, MSG_BURGLARY);
    }

    private void z12()
    {
        log_z("z12", "  ");
        envStopGate(1);
    }

    private void z13()
    {
        log_z("z13", " ");
        envCloseGate(1);
    }

    private void z14()
    {
        log_z("z14", " ");
        envOpenGate(1);
    }

    private void z15()
    {
        log_z("z15", " ");
        envLock(1);
    }

    private void z16()
    {
        log_z("z16", " ");
        envUnlock(1);
    }

    //--- Methods for private usage ---
    private boolean isClosed()
    {
        return (0 == _y3) || (4 == _y3);
    }

    private boolean isLocked()
    {
        return (0 == _y3);
    }

    private boolean needCheck_A3()
    {
        return (1 <= _y3) && (5 >= _y3);
    }
}
