package BSS.Devices;

import BSS.*;

import java.util.Map;

public class ParkingDevice extends BaseDevice
{
    byte _metal_sensor_status;
    String _keyboard_data;

    public ParkingDevice(int id, Logger logger, Map configuration, Communicator communicator)
    {
        super(id, logger, configuration, communicator);
        _metal_sensor_status = 0;
        _keyboard_data = "";
    }

    public String toString()
    {
        return "Parking#" + ID();
    }

    //--- Device interface methods ---
    public byte getStatus(int device_type, int device_id)
    {
        boolean read = false;
        byte value = 0;
        switch (device_type)
        {
          case DEV_METAL_SENSOR:
            if (1 == device_id)
            {
                value = _metal_sensor_status;
                read = true;
            }
          break;
        }
        log_get_status(read, device_type, device_id, value);
        return value;
    }

    public byte[] readData(int device_type, int device_id)
    {
        boolean read = false;
        byte[] value = new byte[0];
        switch (device_type)
        {
          case DEV_KEYBOARD:
            if (1 == device_id)
            {
                value = _keyboard_data.getBytes();
                read = true;
            }
          break;
        }
        log_read_data(read, device_type, device_id, value);
        return value;
    }

    public void writeData(int device_type, int device_id, byte[] data)
    {
        boolean write = false;
        switch (device_type)
        {
          case DEV_PRINTER:
            if (1 == device_id)
            {
                write = true;
            }
          break;
        }
        log_write_data(write, device_type, device_id, data);
    }

    public void sendSMS(SMS sms)
    {
        log(1, _con.toString() + " attempts to send " + sms);
    }

    //--- Host interface methods ---
    public void updateStatus(int device_type, int device_id, byte value)
    {
        boolean updated = false;
        switch (device_type)
        {
          case DEV_METAL_SENSOR:
            if (1 == device_id)
            {
                _metal_sensor_status = value;
                updated = true;
            }
          break;
        }
        log_update_status(updated, device_type, device_id, value);
    }

    public void updateData(int device_type, int device_id, byte[] value)
    {
        boolean updated = false;
        switch (device_type)
        {
          case DEV_KEYBOARD:
            if (1 == device_id)
            {
                _keyboard_data = new String(value);
                updated = true;
            }
          break;
        }
        log_update_data(updated, device_type, device_id, value);
    }

    public void receiveSMS(SMS sms)
    {
        log(1, "Unexpected SMS: " + sms);
    }
}
