package BSS.Devices;

import BSS.*;

import java.util.Map;

public class RoomDevice extends BaseDevice
{
    byte _fire_sensor_status;
    byte _volume_sensor_status;
    byte _lift_status;

    public RoomDevice(int id, Logger logger, Map configuration, Communicator communicator)
    {
        super(id, logger, configuration, communicator);
        _fire_sensor_status = 0;
        _volume_sensor_status = 0;
        _lift_status = 0;
    }

    public String toString()
    {
        return "Room#" + ID();
    }

    //--- Device interface methods ---
    public byte getStatus(int device_type, int device_id)
    {
        boolean read = false;
        byte value = 0;
        switch (device_type)
        {
          case DEV_FIRE_SENSOR:
            if (1 == device_id)
            {
                value = _fire_sensor_status;
                read = true;
            }
          break;

          case DEV_VOLUME_SENSOR:
            if (1 == device_id)
            {
                value = _volume_sensor_status;
                read = true;
            }
          break;

          case DEV_LIFT_CONTROLLER:
            if (1 == device_id)
            {
                value = _lift_status;
                read = true;
            }
          break;
        }
        log_get_status(read, device_type, device_id, value);
        return value;
    }

    public void setStatus(int device_type, int device_id, byte status)
    {
        boolean write = false;
        switch (device_type)
        {
          case DEV_SMUDGE:
          case DEV_CAMERA:
          case DEV_INTERFERENCE:
          case DEV_LIFT_CONTROLLER:
          case DEV_LED:
          case DEV_FIRE_EXTINGUISHER:
            if (1 == device_id)
            {
                write = true;
            }
          break;
        }
        log_set_status(write, device_type, device_id, status);
    }

    public void sendSMS(SMS sms)
    {
        log(1, _con.toString() + " attempts to send " + sms);
    }

    //--- Host interface methods ---
    public void updateStatus(int device_type, int device_id, byte value)
    {
        boolean updated = false;
        switch (device_type)
        {
          case DEV_FIRE_SENSOR:
            if (1 == device_id)
            {
                _fire_sensor_status = value;
                updated = true;
            }
          break;

          case DEV_VOLUME_SENSOR:
            if (1 == device_id)
            {
                _volume_sensor_status = value;
                updated = true;
            }
          break;

          case DEV_LIFT_CONTROLLER:
            if (1 == device_id)
            {
                _lift_status = value;
                updated = true;
            }
          break;
        }
        log_update_status(updated, device_type, device_id, value);
    }

    public void receiveSMS(SMS sms)
    {
        log(1, "Unexpected SMS: " + sms);
    }
}
